
subroutine CalcPDF(Pos, Type, L, Hist, NBins, Dim, NAtom)
    implicit none
    real(8), intent(in) :: L
    integer, intent(in) :: NBins, Dim, NAtom 
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NAtom-1) :: Type
    integer, intent(inout), dimension(0:NBins-1) :: Hist
!f2py intent(in,out,inplace) :: Hist
    real(8), dimension(0:Dim-1) :: Posi, rij
    integer :: Typei, i, j
    real(8) :: BinSize, d
    integer :: BinNum

    BinSize = L/NBins

    do i=1, NAtom-1
        Posi = Pos(i,:)
        Typei = Type(i)
        if(Typei == 0) then
            cycle
        endif
        do j=i+1, NAtom-1
            if(Type(j) == 0) then
                cycle
            endif
            rij = Pos(j,:) - Posi
            rij = rij - L*dnint(rij/L)
            d = sqrt(sum(rij*rij))
            BinNum = int(d/BinSize)
            Hist(BinNum)=Hist(BinNum)+1
        enddo
    enddo
    
end subroutine    

subroutine CalcEnergy(Pos, Type, M, L, cut, sigmaP, sigmaF, epsilonPP, epsilonFF, epsilonPF, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: sigmaP, sigmaF
    real(8), intent(in) :: epsilonPP, epsilonFF, epsilonPF
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(in) :: L, cut
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000.
    real(8) :: r0
    integer :: Typei
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12
    real(8) :: rc, rcP, rcF, rcPF, rcP2, rcF2, rcPF2
    real(8) :: sigmaPF
    real(8) :: ShiftPP, ShiftFF, ShiftPF
    integer :: i,j

    PEnergy = 0.

    sigmaPF = (sigmaP + sigmaF)/2.
    r0 = sigmaP
    rc = cut*sigmaP
    rcP = rc/sigmaP
    rcF = rc/sigmaF
    rcPF = rc/sigmaPF
    
    ShiftPP = -4. * epsilonPP * (rcP**(-12) - rcP**(-6))
    ShiftFF = -4. * epsilonFF * (rcF**(-12) - rcF**(-6))
    ShiftPF = -4. * epsilonPF * (rcPF**(-12) - rcPF**(-6))

    rcP2 = rcP*rcP
    rcF2 = rcF*rcF
    rcPF2 = rcPF*rcPF

    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        Typei = Type(i)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
            if (Type(j) == Typei .and. Typei == 0) then
                !Poly-Poly interaction
                if (j == i+1 .and. mod(j,M) > 0) then
                    !bonded interaction, use harmonic potential
                    d = sqrt(d2)
                    PEnergy = PEnergy + k/2.*epsilonPP*(d-r0)**2
                else
                    !nonbonded interaction, use the LJ potential
                    if(d2 > rcP2) then
                        cycle
                    endif
                    id2 = sigmaP / d2
                    id6 = id2*id2*id2
                    id12 = id6*id6
                    PEnergy = PEnergy + 4.*epsilonPP*(id12-id6) + ShiftPP
                endif
            else if (Type(j) == Typei .and. Typei == 1) then
                !Full-Full interaction
                !nonbonded interaction, use the LJ potential
                if(d2 > rcF2) then
                    cycle
                endif
                id2 = sigmaF / d2
                id6 = id2*id2*id2
                id12 = id6*id6
                PEnergy = PEnergy + 4.*epsilonFF*(id12-id6) + ShiftFF
            else
                !Poly-Full interaction
                !nonbonded interaction, use the LJ potential
                if(d2 > rcPF2) then
                    cycle
                endif
                id2 = sigmaPF / d2
                id6 = id2*id2*id2
                id12 = id6*id6
                PEnergy = PEnergy + 4.*epsilonPF*(id12-id6) + ShiftPF
            endif
        enddo
    enddo
end subroutine

subroutine CalcEnergyForces(Pos,Type,M,L,cut,sigmaP,sigmaF,epsilonPP,epsilonFF,epsilonPF,mP,mF,PEnergy,Forces,Dim,NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: sigmaP, sigmaF
    real(8), intent(in) :: mP, mF
    real(8), intent(in) :: epsilonPP, epsilonFF, epsilonPF
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), intent(in) :: L, cut
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000.
    real(8) :: r0
    integer :: Typei
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d, d2, id2, id6, id12
    real(8) :: rc, rcP, rcF, rcPF, rcP2, rcF2, rcPF2
    real(8) :: sigmaPF
    real(8) :: ShiftPP, ShiftFF, ShiftPF
    integer :: i,j

    PEnergy = 0.
    Forces = 0.

    sigmaPF = (sigmaP + sigmaF)/2.
    r0 = sigmaP
    rc = cut*sigmaP
    rcP = rc/sigmaP
    rcF = rc/sigmaF
    rcPF = rc/sigmaPF
    
    ShiftPP = -4. * epsilonPP * (rcP**(-12) - rcP**(-6))
    ShiftFF = -4. * epsilonFF * (rcF**(-12) - rcF**(-6))
    ShiftPF = -4. * epsilonPF * (rcPF**(-12) - rcPF**(-6))
    rcP2 = rcP*rcP
    rcF2 = rcF*rcF
    rcPF2 = rcPF*rcPF

    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        Typei = Type(i)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
            if (Type(j) == Typei .and. Typei == 0) then
                !Poly-Poly interaction
                if (j == i+1 .and. mod(j,M) > 0) then
                    !bonded interaction, use harmonic potential
                    d = sqrt(d2)
                    PEnergy = PEnergy + k/2.*epsilonPP*(d-r0)**2
                    !check this
                    Fij = rij * (epsilonPP/sigmaP) * (k * (d-r0)/d)
                    Forces(i,:) = Forces(i,:) + mP*Fij
                    Forces(j,:) = Forces(j,:) - mP*Fij
                else
                    !nonbonded interaction, use the LJ potential
                    if(d2 > rcP2) then
                        cycle
                    endif
                    id2 = sigmaP / d2
                    id6 = id2*id2*id2
                    id12 = id6*id6
                    PEnergy = PEnergy + 4.*epsilonPP*(id12-id6) + ShiftPP
                    Fij = rij*(epsilonPP/sigmaP)*((-48. * id12 + 24. * id6) * id2)
                    Forces(i,:) = Forces(i,:) + mP*Fij
                    Forces(j,:) = Forces(j,:) - mP*Fij
                endif
            else if (Type(j) == Typei .and. Typei == 1) then
                !Full-Full interaction
                !nonbonded interaction, use the LJ potential
                if(d2 > rcF2) then
                    cycle
                endif
                id2 = sigmaF / d2
                id6 = id2*id2*id2
                id12 = id6*id6
                PEnergy = PEnergy + 4.*epsilonFF*(id12-id6) + ShiftFF
                Fij = rij*(epsilonFF/sigmaF)*((-48. * id12 + 24. * id6) * id2)
                Forces(i,:) = Forces(i,:) + mF*Fij
                Forces(j,:) = Forces(j,:) - mF*Fij
            else
                !Poly-Full interaction
                !nonbonded interaction, use the LJ potential
                if(d2 > rcPF2) then
                    cycle
                endif
                id2 = sigmaPF / d2
                id6 = id2*id2*id2
                id12 = id6*id6
                PEnergy = PEnergy + 4.*epsilonPF*(id12-id6) + ShiftPF
                Fij = rij*(epsilonPF/sigmaPF)*((-48. * id12 + 24. * id6) * id2)
                if(Typei == 0) then
                    Forces(i,:) = Forces(i,:) + mP*Fij
                    Forces(j,:) = Forces(j,:) - mF*Fij
                else
                    Forces(i,:) = Forces(i,:) + mF*Fij
                    Forces(j,:) = Forces(j,:) - mP*Fij
                endif
            endif
        enddo
    enddo
end subroutine

subroutine VVIntegrate(Pos,Type,Vel,Accel,M,L,rc,sigmaP,sigmaF,epsilonPP,epsilonFF,epsilonPF,mP,mF,dt,KEnergy,PEnergy,Dim,NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(in) :: sigmaP, sigmaF
    real(8), intent(in) :: mP, mF
    real(8), intent(in) :: epsilonPP, epsilonFF, epsilonPF
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
    integer, intent(in), dimension(0:NAtom-1) :: Type
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    integer :: i

!Accel is a misnomer now that mass is involved

    do i=0, NAtom-1
        if(Type(i) == 0) then
            Pos(i,:) = Pos(i,:) + dt * Vel(i,:) + (1/(2*mP)) * dt*dt * Accel(i,:)
            Vel(i,:) = Vel(i,:) + (1/(2*mP)) * dt * Accel(i,:)
        else
            Pos(i,:) = Pos(i,:) + dt * Vel(i,:) + (1/(2*mF)) * dt*dt * Accel(i,:)
            Vel(i,:) = Vel(i,:) + (1/(2*mF)) * dt * Accel(i,:)
        endif
    enddo
    call CalcEnergyForces(Pos,Type,M,L,rc,sigmaP,sigmaF,epsilonPP,epsilonFF,epsilonPF,mP,mF,PEnergy,Accel,Dim,NAtom)

    do i=0, NAtom-1
        if(Type(i) == 0) then
            Vel(i,:) = Vel(i,:) + (1/(2*mP)) * dt * Accel(i,:)
        else
            Vel(i,:) = Vel(i,:) + (1/(2*mF)) * dt * Accel(i,:)
        endif
    enddo
        KEnergy = 0.5 * sum(Vel*Vel)
end subroutine